/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is tkishdemos.
 *
 * The Initial Developer of the Original Code is
 * M.D. Nauta.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


function showSourceCode(filename) {
	window.openDialog('chrome://tkishdemos/content/seeCode.xul',
		filename+'_w',
		'chrome,resizable=yes',
		filename);
}

function addComposingFiles2Popup() {
	var i;
	var xulFile = window.location.pathname.substr(1);
	var popup = document.getElementById('otherFiles');
	var children = popup.childNodes;
	for (i=0; i<children.length; ++i) {
		popup.removeChild(children[i]);
	}
	addPopupMenuitem(popup,xulFile);
	var additionalFiles = getComposingFiles(document);
	additionalFiles = getRelativePaths(additionalFiles);
	additionalFiles = additionalFiles.sort();
	for (i=0; i<additionalFiles.length; ++i) {
		addPopupMenuitem(popup,additionalFiles[i]);
	}
}

function getRelativePaths(files) {
	var packageName = window.location.host;
	var rv = new Array();
	var str,idx;
	for (var i=0; i<files.length; ++i) {
		idx = files[i].indexOf(packageName);
		if (idx == -1) continue; // Remove files from other packages.
		str = files[i].substr(idx+packageName.length+1);
		str = str.replace(/^jar!\//,''); //apparently some uris are changed by packaging.
		rv.push(str.replace(/^locale\//,'locale/en-US/'));
	}
	return rv;
}

function addPopupMenuitem(popup,file) {
	var newElem = document.createElement('menuitem');
	newElem.setAttribute('label',file);
	newElem.setAttribute('oncommand','showSourceCode("' + file + '")');
	popup.appendChild(newElem);
}

function getComposingFiles(doc) {
	// Return an array with all files (css,dtd,js,properties,etc.) that are used in constructing document doc, except for the main xul file (window.location.pathname.substr(1)).
	var files = getStylesheets(doc);
	files = files.concat(getOverlays(doc));
	files.push(getDocType(doc));
	files = files.concat(uriAttributes(doc,'src'));
	files = files.concat(uriAttributes(doc,'data')); // object elements.
	// find all documentNodes and apply this funcion recursively.
	var filter = function (node) {
		if (node.nodeType == Node.ELEMENT_NODE) {
			if (node.contentDocument) return NodeFilter.FILTER_ACCEPT;
			// The treeWalker doesn't descend into subdocs so using NodeFilter.SHOW_DOCUMENT is useless.
			// The inspector/deep-tree-walker implementation seems very incomplete.
			return NodeFilter.FILTER_SKIP;
		}
		return NodeFilter.FILTER_REJECT;
	};
	if (doc.createTreeWalker) {
		var tw = doc.createTreeWalker(doc.documentElement,NodeFilter.SHOW_ELEMENT,filter,false);
		var node;
		while ((node = tw.nextNode())) {
			files = files.concat(getComposingFiles(node.contentDocument));
		}
	}
	return files;
}

function getStylesheets(doc) {
	var stylesheets = doc.styleSheets;
	var rv = new Array();
	for (var i=0; i<stylesheets.length; ++i) {
		var href = stylesheets.item(i).href;
		if (href) rv.push(href);
		// temporary hack
		if (href.search(/Overlay/) != -1) {
			rv.push(href.replace(/\/skin\//,'/content/').replace(/\.css$/,'.xul'));
			rv.push(href.replace(/\/skin\//,'/locale/').replace(/\.css$/,'.dtd'));
		}
		var rules = stylesheets.item(i).cssRules;
		var idx;
		for (var j=0; j<rules.length; ++j) {
			if (rules.item(j).type == CSSRule.IMPORT_RULE) {
				rv.push(rules.item(j).href);
			} else {
				if (rules.item(j).cssText.search(/:\s*url\(([^\)]+)\)/) != -1) {
					if ((idx = RegExp.$1.lastIndexOf('#')) != -1) {
						rv.push(RegExp.$1.substr(0,idx));
					} else {
						rv.push(RegExp.$1);
					}
				}
			}
		}
	}
	return rv;
}

function getOverlays(doc) {
	var rv = new Array();
	// How do you determine the overlays in use? A treeWalker searching for processing instructions doesn't find them.
	// for a temporary hack see getStylesheets because the stylesheet used in an overlay is found and where there is a stylesheet there must be a xul file.
	return rv;
}

function getDocType(doc) {
	var doctype;
	if (doc.doctype) {
		doctype = doc.doctype.systemId; // wishful thinking.
	} else {
		doctype = window.location.href.replace(/\/content\//,'/locale/').replace(/\.xul$/,'.dtd');
	}
	return doctype;
}

function uriAttributes(doc,attr) {
	var root = doc.documentElement;
	var rv = new Array();
	var filter = function (node) {
		if (node.nodeType == Node.ELEMENT_NODE) {
			if (node.hasAttribute(attr)) return NodeFilter.FILTER_ACCEPT;
			return NodeFilter.FILTER_SKIP;
		}
		return NodeFilter.FILTER_REJECT;
	};
	if (doc.createTreeWalker) {
		var tw = doc.createTreeWalker(root,NodeFilter.SHOW_ELEMENT,filter,false);
		var node;
		while ((node = tw.nextNode())) {
			rv.push(node.getAttribute(attr));
		}
	}
	return rv;
}
